#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdlib.h>
#include <string.h>

#include "hd44780_simple.h"

volatile uint16_t ms;
volatile uint8_t t1ovf;

inline void timers_start(void)
{
	ms=0;

	TCNT0 = 0;
	OCR0 = 250-1;	// 16MHz/64/250 = 1kHz
	SFIOR |= 1;		// reset the prescaler

	t1ovf = 0;
	TCNT1 = 0;

	sei();

	TCCR1B = 7;				// start timer1, external clock on T1
	TCCR0 = (1<<WGM01) | 3;	// start timer0, CTC mode, prescaler=64
}

inline void timers_stop(void)
{
	TCCR1B = 0;
	TCCR0 = 0;
	cli();
}

void update_display(uint16_t scale)
{
	int32_t cval;
	char buf1[16];
	char buf2[16];

	cval = (((uint32_t)t1ovf<<16) + TCNT1)*scale;

	cval -= (cval/1000000)*3*scale;	// subtract error: 50 clocks / 16 MHz = 3.125us

	ltoa(cval, buf1, 10);
	strcpy(buf2,"               ");
	strcpy(buf2+(8-strlen(buf1)),buf1);
	memcpy(buf2+8," Hz",3);
	lcd_goto(0,1);
	lcd_puts(buf2);
}
	

int main(void)
{
	int32_t cval;
	uint16_t ms_temp;

	DDRB |= 1;	// led on PB0

	lcd_init();
	lcd_puts_P(PSTR("**Freq Counter**"));

	TIMSK = (1<<OCIE0) | (1<<TOIE1);	// enable t0 compare int and t1 ovf int

	for(;;) {
		PORTB ^=(1<<PB0);	// blink led

		/* 100 ms gate time */
		timers_start();

		do
			{ cli(); ms_temp=ms; sei();}	// make sure read is atomic
		while(ms_temp<100);					// this takes 9 clocks

		timers_stop();

		cval = ((uint32_t)t1ovf<<16) + TCNT1;

		/* use 1000ms gate time for low frequency signals */
		if(cval<100000) {
			timers_start();

			do {
				cli(); ms_temp=ms; sei();	// make sure read is atomic
			} while(ms_temp<1000);			// this takes 9 clocks

			timers_stop();
			update_display(1);
			continue;
		}
		update_display(10);
	}
}

/* millisecond interrupt */
/* this takes 41 clocks */
ISR(TIMER0_COMP_vect)
{
	ms++;
}

/* this is the counter */
ISR(TIMER1_OVF_vect)
{
	t1ovf++;
}
