/**************************************************
*                HD44780 LCD Driver               *
*                  Copyright 2010                 *
*           Marko Borsic - glagnar@gmail.com      *
**************************************************/

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>

#include "hd44780_simple.h"

/*
** write a byte to the LCD, either as a command or data
*/
void lcd_wr(uint8_t reg, uint8_t c)
{
	if (reg==LCD_CMD) LCD_PORT &= ~LCD_RS; // RS = 0 -> command

	LCD_PORT &= 0xf0;			// clear data bits
	LCD_PORT |= (c & 0xf0)>>4;	// output high nibble

	LCD_PORT |= LCD_E;			// strobe E high
	_delay_us(40);
	LCD_PORT &= ~LCD_E;			// take E low again

	LCD_PORT &= 0xf0;			// clear data bits
	LCD_PORT |= (c & 0x0f);		// output low nibble

	LCD_PORT |= LCD_E;			// strobe E high
	_delay_us(40);
	LCD_PORT &= ~LCD_E;			// take E low again

	if (reg==LCD_CMD) LCD_PORT |= LCD_RS; // set RS again for data mode
}


/*
** change cursor position to x,y
*/
void lcd_goto(uint8_t x, uint8_t y)
{
	switch(y) {
		case 0: y = LCD_LINE1; break;
#if LCD_LINES > 1
		case 1: y = LCD_LINE2; break;
#endif
#if LCD_LINES > 2
		case 2: y = LCD_LINE3; break;
#endif
#if LCD_LINES > 3
		case 3: y = LCD_LINE4; break;
#endif
		default: return;
	}
	lcd_wr(LCD_CMD,0x80 + y + x);
}


/*
** clear the LCD and also the buffer
*/
void lcd_clear(void)
{
	lcd_wr(LCD_CMD,LCD_CLEAR);
	_delay_ms(2);
}

	
/*
** initialization sequence
*/
void lcd_init(void)
{
	LCD_PORT |= (0x0F | LCD_E | LCD_RS);
	LCD_DDR |= (0x0F | LCD_E | LCD_RS);

	LCD_PORT &= ~LCD_E;

	_delay_ms(20);
	lcd_wr(LCD_CMD,0x33); // 0x30, 0x30
	_delay_ms(5);
	lcd_wr(LCD_CMD,0x32); // 0x30, 0x20
	_delay_us(150);

	lcd_wr(LCD_CMD,FUNCTION_SET);
	_delay_ms(1);
	lcd_wr(LCD_CMD,FUNCTION_SET);
	_delay_ms(1);
	lcd_wr(LCD_CMD,LCD_CURSOR_OFF);
	_delay_ms(1);
	lcd_wr(LCD_CMD,LCD_CLEAR);
	_delay_ms(2);
	lcd_wr(LCD_CMD,LCD_HOME_L1);
	_delay_ms(2);

	lcd_clear();
}


/*
** print a string to the lcd
*/
void lcd_puts(char *s)
{
	while(*s)
		lcd_wr(LCD_DATA,*s++);
}


/*
** print a PROGMEM string to the lcd
*/
void lcd_puts_P(const char *s)
{
	char c;

	while((c = pgm_read_byte(s++)))
		lcd_wr(LCD_DATA,c);
}

/*
** write data to the CGRAM
*/
void lcd_load_cgram(uint8_t addr, uint8_t * data, uint8_t len)
{
	uint8_t i;

	lcd_wr(LCD_CMD,LCD_CGRAM_ADDR + addr/8);

	for (i = 0; i < len; i++)
		lcd_wr(LCD_DATA,pgm_read_byte(&data[i]));
}
