#ifndef _HD44780_SIMPLE_H
#define _HD44780_SIMPLE_H

/* change these lines to adapt to your display and uC */

#define LCD_LINES 2	// number of lines on the display
#define LCD_COLS 16	// LCD line length

/* R/W is connected to GND */
#define LCD_PORT PORTA	// we're using the lower nibble
#define LCD_DDR DDRA
#define LCD_E (1<<PA4)
#define LCD_RS (1<<PA5)

/* user defines end here */

#define LCD_LINE1 0U
#if LCD_LINES > 1
#define LCD_LINE2 0x40U
#endif

#if LCD_LINES == 4 && LCD_COLS == 20
#define LCD_LINE3 0x14U
#define LCD_LINE4 0x54U

#elif LCD_LINES == 4 && LCD_COLS == 16
#define LCD_LINE3 0x10U
#define LCD_LINE4 0x50U
#endif

#define LCD_CMD 0
#define LCD_DATA 1

/* HD44780 commands */
#define LCD_CLEAR        0x01    // clear LCD display and home cursor
#define CURSOR_MODE_DEC  0x04    // cursor auto decrement after R/W
#define CURSOR_MODE_INC  0x06    // cursor auto increment after R/W
#define FUNCTION_SET     0x28    // setup, 4 bits, 2 lines, 5X7
#define LCD_CURSOR_ON    0x0E    // display on with cursor
#define LCD_CURSOR_OFF   0x0C    // display on with cursor off
#define LCD_CURSOR_BLINK 0x0D    // display on with blinking cursor
#define LCD_CURSOR_LEFT  0x10    // move cursor left one position
#define LCD_CURSOR_RIGHT 0x14    // move cursor right one position
#define LCD_HOME_L1	     0x80	 // set cursor to line 0, col 0
#define LCD_OFF			 0x08	 // display off
#define LCD_CGRAM_ADDR	 0x40	 // set the CGRAM address

void lcd_init(void);
void lcd_clear(void);
void lcd_wr(uint8_t reg, uint8_t c);
void lcd_goto(uint8_t x, uint8_t y);
void lcd_putc(char c);
void lcd_puts(char *s);
void lcd_puts_P(const char *s);
void lcd_load_cgram(uint8_t addr, uint8_t * data, uint8_t len);

#endif
